@@ -20,7 +20,7 @@
  */
 
 #define CLOOP_NAME "cloop"
-#define CLOOP_VERSION "2.639"
+#define CLOOP_VERSION "3.14"
 #define CLOOP_MAX 8
 
 #ifndef KBUILD_MODNAME
@@ -164,22 +164,26 @@ static int cloop_count = 0;
  * 2MB in one piece */
 static void *cloop_malloc(size_t size)
 {
- int order = get_order(size);
- if(order <= KMALLOC_MAX_ORDER)
-   return (void *)kmalloc(size, GFP_KERNEL);
- else if(order < MAX_ORDER)
-   return (void *)__get_free_pages(GFP_KERNEL, order);
+ /* kmalloc will fail after the system is running for a while, */
+ /* when large orders can't return contiguous memory. */
+ /* Let's just use vmalloc for now. :-/ */
+ /* int order = get_order(size); */
+ /* if(order <= KMALLOC_MAX_ORDER) */
+ /*  return (void *)kmalloc(size, GFP_KERNEL); */
+ /* else if(order < MAX_ORDER) */
+ /*  return (void *)__get_free_pages(GFP_KERNEL, order); */
  return (void *)vmalloc(size);
 }
 
 static void cloop_free(void *mem, size_t size)
 {
- int order = get_order(size);
- if(order <= KMALLOC_MAX_ORDER)
-   kfree(mem);
- else if(order < MAX_ORDER)
-   free_pages((unsigned long)mem, order);
- else vfree(mem);
+ /* int order = get_order(size); */
+ /* if(order <= KMALLOC_MAX_ORDER) */
+ /*  kfree(mem); */
+ /* else if(order < MAX_ORDER) */
+ /*  free_pages((unsigned long)mem, order); */
+ /* else */
+ vfree(mem);
 }
 
 static int uncompress(struct cloop_device *clo,
@@ -292,12 +296,21 @@ static int cloop_handle_request(struct cloop_device *clo, struct request *req)
  int buffered_blocknum = -1;
  int preloaded = 0;
  loff_t offset     = (loff_t) blk_rq_pos(req)<<9; /* req->sector<<9 */
- struct bio_vec *bvec;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
+struct bio_vec *bvec;
+#else 
+ struct bio_vec bvec;
+#endif
  struct req_iterator iter;
  rq_for_each_segment(bvec, req, iter)
   {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
    unsigned long len = bvec->bv_len;
    char *to_ptr      = kmap(bvec->bv_page) + bvec->bv_offset;
+#else
+   unsigned long len = bvec.bv_len;
+   char *to_ptr      = kmap(bvec.bv_page) + bvec.bv_offset;
+#endif
    while(len > 0)
     {
      u_int32_t length_in_buffer;
@@ -337,7 +350,11 @@ static int cloop_handle_request(struct cloop_device *clo, struct request *req)
      len         -= length_in_buffer;
      offset      += length_in_buffer;
     } /* while inner loop */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
    kunmap(bvec->bv_page);
+#else
+   kunmap(bvec.bv_page);
+#endif
   } /* end rq_for_each_segment*/
  return ((buffered_blocknum!=-1) || preloaded);
 }
@@ -743,7 +760,7 @@ static int cloop_get_status(struct cloop_device *clo,
  struct kstat stat;
  int err;
  if (!file) return -ENXIO;
- err = vfs_getattr(file->f_path.mnt, file->f_path.dentry, &stat);
+ err = vfs_getattr(&file->f_path, &stat);
  if (err) return err;
  memset(info, 0, sizeof(*info));
  info->lo_number  = clo->clo_number;
@@ -921,14 +938,13 @@ static int cloop_open(struct block_device *bdev, fmode_t mode)
  return 0;
 }
 
-static int cloop_close(struct gendisk *disk, fmode_t mode)
+static void cloop_close(struct gendisk *disk, fmode_t mode)
 {
- int cloop_num, err=0;
- if(!disk) return 0;
+ int cloop_num;
+ if(!disk) return;
  cloop_num=((struct cloop_device *)disk->private_data)->clo_number;
- if(cloop_num < 0 || cloop_num > (cloop_count-1)) return 0;
+ if(cloop_num < 0 || cloop_num > (cloop_count-1)) return;
  cloop_dev[cloop_num]->refcnt-=1;
- return err;
 }
 
 static struct block_device_operations clo_fops =
